<?php

namespace Larajobs\Company\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Larajobs\Company\Http\Requests\ApplicationRequest;
use Larajobs\Company\Interfaces\ApplicationRepositoryInterface;
use Larajobs\Company\Models\Application;

/**
 * Resource controller class for application.
 */
class ApplicationResourceController extends BaseController
{

    /**
     * Initialize application resource controller.
     *
     * @param type ApplicationRepositoryInterface $application
     *
     * @return null
     */
    public function __construct(ApplicationRepositoryInterface $application)
    {
        parent::__construct();
        $this->repository = $application;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Larajobs\Company\Repositories\Criteria\ApplicationResourceCriteria::class);
    }

    /**
     * Display a list of application.
     *
     * @return Response
     */
    public function index(ApplicationRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Larajobs\Company\Repositories\Presenter\ApplicationPresenter::class)
                ->$function();
        }

        $applications = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company::application.names'))
            ->view('company::application.index', true)
            ->data(compact('applications', 'view'))
            ->output();
    }

    /**
     * Display application.
     *
     * @param Request $request
     * @param Model   $application
     *
     * @return Response
     */
    public function show(ApplicationRequest $request, Application $application)
    {

        if ($application->exists) {
            $view = 'company::application.show';
        } else {
            $view = 'company::application.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company::application.name'))
            ->data(compact('application'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new application.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ApplicationRequest $request)
    {

        $application = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::application.name')) 
            ->view('company::application.create', true) 
            ->data(compact('application'))
            ->output();
    }

    /**
     * Create new application.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ApplicationRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $application                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::application.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/application/' . $application->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/application'))
                ->redirect();
        }

    }

    /**
     * Show application for editing.
     *
     * @param Request $request
     * @param Model   $application
     *
     * @return Response
     */
    public function edit(ApplicationRequest $request, Application $application)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::application.name'))
            ->view('company::application.edit', true)
            ->data(compact('application'))
            ->output();
    }

    /**
     * Update the application.
     *
     * @param Request $request
     * @param Model   $application
     *
     * @return Response
     */
    public function update(ApplicationRequest $request, Application $application)
    {
        try {
            $attributes = $request->all();

            $application->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::application.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/application/' . $application->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/application/' . $application->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the application.
     *
     * @param Model   $application
     *
     * @return Response
     */
    public function destroy(ApplicationRequest $request, Application $application)
    {
        try {

            $application->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::application.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company/application/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/application/' . $application->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple application.
     *
     * @param Model   $application
     *
     * @return Response
     */
    public function delete(ApplicationRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::application.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company/application'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/application'))
                ->redirect();
        }

    }

    /**
     * Restore deleted applications.
     *
     * @param Model   $application
     *
     * @return Response
     */
    public function restore(ApplicationRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company::application.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company/application'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/application/'))
                ->redirect();
        }

    }

}
