<?php

namespace Larajobs\Company\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Larajobs\Company\Http\Requests\EmployerRequest;
use Larajobs\Company\Interfaces\EmployerRepositoryInterface;
use Larajobs\Company\Models\Employer;

/**
 * Resource controller class for employer.
 */
class EmployerResourceController extends BaseController
{

    /**
     * Initialize employer resource controller.
     *
     * @param type EmployerRepositoryInterface $employer
     *
     * @return null
     */
    public function __construct(EmployerRepositoryInterface $employer)
    {
        parent::__construct();
        $this->repository = $employer;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Larajobs\Company\Repositories\Criteria\EmployerResourceCriteria::class);
    }

    /**
     * Display a list of employer.
     *
     * @return Response
     */
    public function index(EmployerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Larajobs\Company\Repositories\Presenter\EmployerPresenter::class)
                ->$function();
        }

        $employers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('company::employer.names'))
            ->view('company::employer.index', true)
            ->data(compact('employers', 'view'))
            ->output();
    }

    /**
     * Display employer.
     *
     * @param Request $request
     * @param Model   $employer
     *
     * @return Response
     */
    public function show(EmployerRequest $request, Employer $employer)
    {

        if ($employer->exists) {
            $view = 'company::employer.show';
        } else {
            $view = 'company::employer.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('company::employer.name'))
            ->data(compact('employer'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new employer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(EmployerRequest $request)
    {

        $employer = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('company::employer.name')) 
            ->view('company::employer.create', true) 
            ->data(compact('employer'))
            ->output();
    }

    /**
     * Create new employer.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(EmployerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $employer                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('company::employer.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/employer/' . $employer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/company/employer'))
                ->redirect();
        }

    }

    /**
     * Show employer for editing.
     *
     * @param Request $request
     * @param Model   $employer
     *
     * @return Response
     */
    public function edit(EmployerRequest $request, Employer $employer)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('company::employer.name'))
            ->view('company::employer.edit', true)
            ->data(compact('employer'))
            ->output();
    }

    /**
     * Update the employer.
     *
     * @param Request $request
     * @param Model   $employer
     *
     * @return Response
     */
    public function update(EmployerRequest $request, Employer $employer)
    {
        try {
            $attributes = $request->all();

            $employer->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('company::employer.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('company/employer/' . $employer->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/employer/' . $employer->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the employer.
     *
     * @param Model   $employer
     *
     * @return Response
     */
    public function destroy(EmployerRequest $request, Employer $employer)
    {
        try {

            $employer->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::employer.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('company/employer/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('company/employer/' . $employer->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple employer.
     *
     * @param Model   $employer
     *
     * @return Response
     */
    public function delete(EmployerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('company::employer.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('company/employer'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/employer'))
                ->redirect();
        }

    }

    /**
     * Restore deleted employers.
     *
     * @param Model   $employer
     *
     * @return Response
     */
    public function restore(EmployerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('company::employer.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/company/employer'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/company/employer/'))
                ->redirect();
        }

    }

}
